allocate=function(grid,allocations,verbose=FALSE){

# get dimension of input grid
nc=ncol(grid)
nr=nrow(grid)

# transform to i,j,v form:
ijv = cbind(rep(1:nr,nc),rep(1:nc,rep(nr,nc)),as.vector(grid))

# set up the output grid
outgrid = matrix(NA,nrow=nr, ncol=nc)

# and the output data frame
outdata=matrix(ncol=5)

# this is slightly bigger than the diagonal of the grid:
max = sqrt(1 + nr^2 + nc^2)

# first allocation is 1
i=1

# now keep going until we run out of allocations:
while(sum(ijv[,3],na.rm=TRUE) > allocations[1]){


# find the highest point on the grid
maxpt = ijv[which.max(ijv[,3]),]

if(verbose)print(maxpt)

# compute the distances from highest point to all others
d = sqrt( (ijv[,1]-maxpt[1])^2 + (ijv[,2]-maxpt[2])^2 )

# do binary search of the input grid up to the allocation:
lupf = bsearch(0,max,capture,allocations[1],0.001,d,ijv,verbose=verbose)

# get the radius
r=lupf[2]

# now find out which unallocated cells we just found
inIt = d<=r & ijv[,3]>0

# allocate them to region i
outgrid[ijv[inIt,1:2]] = i

# total allocation for this region
sumI=sum(ijv[inIt,3],na.rm=TRUE)

cat("sequence: ", i, " radius: ", r," allocations: ", sumI, "\n", sep="")


# set to zero so they don't get used again:
ijv[inIt,3]=0

# add to output dataframe
outdata=rbind(outdata,c(i,maxpt[1],maxpt[2],r,sumI))

# move on to next allocation:
allocations=allocations[-1]

# but if that just trimmed off the last one, we're done:
if(length(allocations)==0)break

# next region is i+1
i=i+1
}

# chop off the initial NAs from the matrix and convert to data frame:
outdata=data.frame(outdata[-1,])

# give it some nice names:
names(outdata)=c("i","x","y","r","a")

# return output data and the assignment grid:
return(list(data = outdata, grid=matrix(outgrid,nrow=nr,ncol=nc)))

}
